//This should contain plugin information and declarations we need throughout the entire plugin

//NOTE:	I know this has nothing to do with plugin information, but this seems like the one place
//		every person is likely to read......
//		You need to make sure that you edit your Makefile or your MSVC project settings so that
//		the proper "include" directories get used. In MSVC you can change your include directories
//		by going to Project->Settings, change the "Settings For" to "All Configurations", click
//		the C/C++ tab, change the category to "Preprocessor", then use "Additional include directories"

//		You'll need to include the metamod source code directory, then the following from the 
//		multiplayer directory of the Half-Life SDK: common,engine,dlls,pm_shared

#ifdef SETTING_UP_CVARS_AND_COMMANDS

//USE THIS AREA TO SETUP:
//1. CVARS
//2. Serverside Commands
//3. Client Commands

//CVARS
//syntax: DEFINE_CVAR(cvar name, cvar flags, cvar default string value, cvar default float value );
//EXAMPLE: DEFINE_CVAR(mp_friendlyfire, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_PRINTABLEONLY , "0", 0 );
//	the cvar would then be accessible as a cvar_t* named cvar_mp_friendlyfire
DEFINE_CVAR(marineredeem_health, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_PRINTABLEONLY , "40", 40 );
DEFINE_CVAR(marineredeem_chance, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_PRINTABLEONLY , "75", 75 );
//DEFINE_CVAR(marineredeem_blockdigest, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_PRINTABLEONLY , "1", 1 );


//SERVER COMMANDS
//DEFINE_SERVER_COMMAND( exampleServerCommand );
//	be sure to create the function body using the example in servercommands.cpp



//CLIENT COMMANDS
//DEFINE_CLIENT_COMMAND( exampleClientCommand );
//	be sure to create the function body using the example in clientcommands.cpp




#else //Don't use DEFINE_CVAR, DEFINE_SERVER_COMMAND, or DEFINE_CLIENT_COMMAND beyond this point, or things break

#ifndef _PLUGIN_H_
#define _PLUGIN_H_

//=================================================================
//Plugin Information
//=================================================================
#define MY_DATE			"August 29, 2003"
#define MY_NAME			"Marine Redeem"
#define MY_AUTHOR		"[WHO]Them"
#define MY_EMAIL		"them@clanwho.com"
#define MY_URL			"www.clanwho.com"
#define MY_LOGTAG		"MARINEREDEEM"
#define MY_VERSION		"1.1"
#define MY_LOADABLE		PT_ANYTIME
#define MY_UNLOADABLE	PT_ANYTIME


#define MY_VERS_DWORD	1,1,0,0
#define MY_COMMENTS		"Muahahahaha"
#define MY_DESC			"Redemption for marines"
#define MY_FILENAME		MY_LOGTAG ".DLL"
#define MY_INTERNAL		MY_LOGTAG
#define MY_COPYRIGHT	"GNU Public License"


#ifndef VERS_PLUGIN_H //so that we don't confuse the resource compiler
#include <extdll.h>		//always

#include <string.h> //to prevent linux errors while processing meta_api.h
#include <meta_api.h>

#include "utilfunctions.h"

#define CREATE_CVAR_EXTERNS
	#include "complexmacros.h"	
#undef CREATE_CVAR_EXTERNS

struct PhaseLL
{
	PhaseLL *next;
	Vector vecPosition;
	bool bPassedTest;
	PhaseLL( void ) : next(NULL), bPassedTest(false) { };
};

//=================================================================
//an EXTREMELY useful setup for dynamically sized data
//=================================================================
struct DynDataAutoCleanUp { 
	bool isInitialized; //used for easy initialization constancy
	
	float *fLastHealth;

	PhaseLL *pHead;

	//float *fExampleOfDynamicData;

	//set default values
	DynDataAutoCleanUp( void ) {
		isInitialized = false;

		fLastHealth = NULL;
		pHead = NULL;

		//fExampleOfDynamicData = NULL; //set pointers to NULL like a good coder :P
	}

	//the entire reason this class exists, auto-cleanup
	~DynDataAutoCleanUp( void ) { //guaranteed to be called when the plugin is being unloaded
		if(isInitialized) {
			isInitialized = false; //just in case

			delete []fLastHealth;
			fLastHealth = NULL;

			PhaseLL *pTemp;
			while(pHead != NULL)
			{
				pTemp = pHead;
				pHead = pHead->next;
				delete pTemp;
			}

			//delete []fExampleOfDynamicData;
			//fExampleOfDynamicData = NULL;
		}
	}

	void Initialize( void ) {
		if(isInitialized) return; //return if we're already initialized
		if( gpGlobals->maxClients == 0 ) return; //since most of the time the dynamically sized data is based on players

		//fExampleOfDynamicData = new float [gpGlobals->maxClients + 1]; //add 1 because it's just easier to use 1 through maxplayers than 0 through maxplayers-1 (less on-the-fly arithmetic)
		
		fLastHealth = new float [gpGlobals->maxClients + 1];

		//initialize the data to default values
		for(int playercounter = 0; playercounter <= gpGlobals->maxClients; ++playercounter) {
			//fExampleOfDynamicData[playercounter] = 0.0f;
			fLastHealth[playercounter] = 0.0f;
		}

		isInitialized = true; //so we don't initialize more than once :P
	}
};
extern DynDataAutoCleanUp DynamicSizedData; //make a global across all files of the above struct, actual declaration in plugin.cpp


//=================================================================
//helper functions
//=================================================================
void plugin_init(void);
void plugin_quit(void);

//simplify the process of registering a user message without stepping on anyone else's code unless we have to
inline void SAFE_USER_MSG(int &gmsgvar, const char *szMsgName, const int &iSize) {
	if(gmsgvar == 0) 
	{
		//attempt to get the msg from the game that's running
		gmsgvar = GET_USER_MSG_ID( &Plugin_info, szMsgName, NULL);
		
		//and if that fails, register it ourselves
		if(gmsgvar == 0) gmsgvar = REG_USER_MSG(szMsgName, iSize);
	}
}




#endif // #ifndef VERS_PLUGIN_H
#endif // #ifndef _PLUGIN_H_
#endif // #ifdef SETTING_UP_CVARS_AND_COMMANDS



